@@ -1,17 +1,16 @@
 import math
-from typing import Dict, Optional, Union, TypeVar, Any
+from typing import Dict, Optional, Union, TypeVar, Any, Type
+from collections.abc import Mapping
 
 import numpy as np
+from pint import DimensionalityError
 
 from euphonic.validate import _check_constructor_inputs
 from euphonic.io import _obj_from_json_file, _obj_to_dict, _process_dict
 from euphonic.readers import castep, phonopy
 from euphonic.util import (direction_changed, is_gamma, get_reference_data)
 from euphonic import (ureg, Quantity, Crystal, DebyeWaller, QpointFrequencies,
-                      StructureFactor)
-
-
-T = TypeVar('T', bound='QpointPhononModes')
+                      StructureFactor, Spectrum1DCollection)
 
 
 class QpointPhononModes(QpointFrequencies):
@@ -37,10 +36,11 @@ class QpointPhononModes(QpointFrequencies):
         Shape (n_qpts, 3*crystal.n_atoms, crystal.n_atoms, 3) complex
         ndarray. The dynamical matrix eigenvectors
     """
+    T = TypeVar('T', bound='QpointPhononModes')
 
     def __init__(self, crystal: Crystal, qpts: np.ndarray,
-                 frequencies: Quantity, eigenvectors: Quantity,
-                 weights: Optional[np.ndarray] = None):
+                 frequencies: Quantity, eigenvectors: np.ndarray,
+                 weights: Optional[np.ndarray] = None) -> None:
         """
         Parameters
         ----------
@@ -197,13 +197,20 @@ class QpointPhononModes(QpointFrequencies):
         Notes
         -----
 
-        This function calculates :math:`|F(Q, \\nu)|^2` per unit cell, where
-        :math:`F(Q, \\nu)` is defined as [1]_:
+        This method calculates the mode-resolved (not binned in energy)
+        one-phonon neutron scattering function
+        :math:`S(Q, \\omega_{q\\nu})` **per atom**, as defined in [1]_. Note
+        that internally Euphonic uses atomic units so :math:`\\hbar` has been
+        omitted from the formulation:
 
         .. math::
 
-          F(Q, \\nu) = \\frac{b_\\kappa}{M_{\\kappa}^{1/2}\\omega_{q\\nu}^{1/2}} \\
-          [Q\\cdot\\epsilon_{q\\nu\\kappa\\alpha}]e^{iQ{\\cdot}r_\\kappa}e^{-W}
+          S(Q, \\omega_{q\\nu}) =
+              \\frac{1}{2N_{atom}} \\
+              \\left\\lvert \\
+              \\sum\\limits_\\kappa\\frac{b_\\kappa}{M_{\\kappa}^{1/2}\\omega_{q\\nu}^{1/2}} \\
+              [Q\\cdot\\epsilon_{q\\nu\\kappa\\alpha}]e^{iQ{\\cdot}r_\\kappa}e^{-W} \\
+              \\right\\rvert^2
 
         Where :math:`\\nu` runs over phonon modes, :math:`\\kappa` runs
         over atoms, :math:`\\alpha` runs over the Cartesian directions,
@@ -212,9 +219,9 @@ class QpointPhononModes(QpointFrequencies):
         vector to atom :math:`\\kappa` in the unit cell,
         :math:`\\epsilon_{q\\nu\\kappa\\alpha}` are the eigevectors,
         :math:`\\omega_{q\\nu}` are the frequencies and :math:`e^{-W}`
-        is the Debye-Waller factor. Note that a factor N for the
-        number of unit cells in the sample hasn't been included, so the
-        returned structure factor is per unit cell.
+        is the Debye-Waller factor. :math:`N_{atom}` is the number of
+        atoms in the unit cell, so the returned structure factor is
+        **per atom** of sample.
 
         .. [1] M.T. Dove, Structure and Dynamics, Oxford University Press, Oxford, 2003, 225-226
 
@@ -225,6 +232,10 @@ class QpointPhononModes(QpointFrequencies):
                 physical_property='coherent_scattering_length')
         elif isinstance(scattering_lengths, dict):
             scattering_length_data = scattering_lengths
+        else:
+            raise TypeError((
+                f'Unexpected type for scattering_lengths, should be str '
+                f'or dict, got {type(scattering_lengths)}'))
 
         sl = [scattering_length_data[x].to('bohr').magnitude
               for x in self.crystal.atom_type]
@@ -265,10 +276,11 @@ class QpointPhononModes(QpointFrequencies):
         # Take mod squared and divide by frequency to get intensity
         sf = np.real(
             np.absolute(term*np.conj(term))/np.absolute(self._frequencies))
+        sf /= 2*self.crystal.n_atoms
 
         return StructureFactor(
             self.crystal, self.qpts, self.frequencies,
-            sf*ureg('bohr**2').to(self.crystal.cell_vectors.units**2),
+            sf*ureg('bohr**2').to('mbarn'),
             temperature=temperature)
 
     def calculate_debye_waller(
@@ -309,18 +321,20 @@ class QpointPhononModes(QpointFrequencies):
 
         .. math::
 
-          e^{-W} = e^{-\\sum_{\\alpha\\beta}{W^{\\kappa}_{\\alpha\\beta}Q_{\\alpha}Q_{\\beta}}}
+          e^{-W} = e^{-\\sum\\limits_{\\alpha\\beta}{W^{\\kappa}_{\\alpha\\beta}Q_{\\alpha}Q_{\\beta}}}
 
         The Debye-Waller exponent is defined as
         :math:`W^{\\kappa}_{\\alpha\\beta}` and is independent of Q, so
         for efficiency can be precalculated to be used in the structure
-        factor calculation. The Debye-Waller exponent is calculated by [2]_
+        factor calculation. The Debye-Waller exponent is calculated by [2]_.
+        Note that internally Euphonic uses atomic units so :math:`\\hbar`
+        has been omitted from the formulation:
 
         .. math::
 
           W^{\\kappa}_{\\alpha\\beta} =
-          \\frac{1}{4M_{\\kappa}\\sum_{q}{weight_q}}
-          \\sum_{q\\nu}weight_q\\frac{\\epsilon_{q\\nu\\kappa\\alpha}\\epsilon^{*}_{q\\nu\\kappa\\beta}}
+          \\frac{1}{4M_{\\kappa}\\sum\\limits_{q}{weight_q}}
+          \\sum\\limits_{q\\nu}weight_q\\frac{\\epsilon_{q\\nu\\kappa\\alpha}\\epsilon^{*}_{q\\nu\\kappa\\beta}}
           {\\omega_{q\\nu}}
           coth(\\frac{\\omega_{q\\nu}}{2k_BT})
 
@@ -390,6 +404,129 @@ class QpointPhononModes(QpointFrequencies):
         dw = dw*ureg('bohr**2').to(self.crystal.cell_vectors_unit + '**2')
         return DebyeWaller(self.crystal, dw, temperature)
 
+    def calculate_pdos(
+            self, dos_bins: Quantity,
+            mode_widths: Optional[Quantity] = None,
+            mode_widths_min: Quantity = Quantity(0.01, 'meV'),
+            weighting: Optional[str] = None,
+            cross_sections: Union[str, Dict[str, Quantity]] = 'BlueBook',
+            ) -> Spectrum1DCollection:
+        """
+        Calculates partial density of states for each atom in the unit
+        cell.
+
+        Parameters
+        ----------
+        dos_bins
+            Shape (n_e_bins + 1,) float Quantity. The energy bin edges
+            to use for calculating the DOS.
+        mode_widths
+            Shape (n_qpts, n_branches) float Quantity in energy units.
+            The broadening width for each mode at each q-point, for
+            adaptive broadening.
+        mode_widths_min
+            Scalar float Quantity in energy units. Sets a lower limit on
+            the mode widths, as mode widths of zero will result in
+            infinitely sharp peaks.
+        weighting
+            One of {'coherent', 'incoherent', 'coherent-plus-incoherent'}.
+            If provided, produces a neutron-weighted DOS, weighted by
+            either the coherent, incoherent, or sum of coherent and
+            incoherent neutron scattering cross-sections.
+        cross_sections
+            A dataset of cross-sections for each element in the structure,
+            it can be a string specifying a dataset, or a dictionary
+            explicitly giving the cross-sections for each element.
+
+            If cross_sections is a string, it is passed to the ``collection``
+            argument of :obj:`euphonic.util.get_reference_data()`. This
+            collection must contain the 'coherent_cross_section' or
+            'incoherent_cross_section' physical property, depending on
+            the ``weighting`` argument. If ``weighting`` is None, this
+            string argument is not used.
+
+            If cross sections is a dictionary, the ``weighting`` argument is
+            ignored, and these cross-sections are used directly to calculate
+            the neutron-weighted DOS. It must contain a key for each element
+            in the structure, and each value must be a Quantity in the
+            appropriate units, e.g::
+
+                {'La': 8.0*ureg('barn'), 'Zr': 9.5*ureg('barn')}
+
+        Returns
+        -------
+        dos
+            A collection of spectra, with the energy bins on the x-axis and
+            PDOS for each atom in the unit cell on the y-axis. If weighting
+            is None, the y-axis is in 1/energy units. If weighting is
+            specified or cross_sections are supplied, the y-axis
+            is in area/energy units per average atom.
+        """
+        weighting_opts = [None, 'coherent', 'incoherent',
+                          'coherent-plus-incoherent']
+        if not weighting in weighting_opts:
+            raise ValueError(f'Invalid value for weighting, got '
+                             f'{weighting}, should be one of '
+                             f'{weighting_opts}')
+
+        cross_sections_data = None
+        if isinstance(cross_sections, str):
+            if weighting is not None:
+                if weighting == 'coherent-plus-incoherent':
+                    weights = ['coherent', 'incoherent']
+                else:
+                    weights = [weighting]
+                cross_sections_data = [get_reference_data(
+                    collection=cross_sections,
+                    physical_property=f'{weight}_cross_section')
+                                       for weight in weights]
+        elif isinstance(cross_sections, Mapping):
+            cross_sections_data = [cross_sections]
+        else:
+            raise TypeError(f'Unexpected type for cross_sections, expected '
+                            f'str or dict, got {type(cross_sections)}')
+
+        if cross_sections_data is not None:
+            cs = [cross_sections_data[0][x] for x in self.crystal.atom_type]
+            if len(cross_sections_data) == 2:
+                cs2 = [cross_sections_data[1][x]
+                        for x in self.crystal.atom_type]
+                cs = [sum(x) for x in zip(cs, cs2)]
+            # Account for cross sections in different, or invalid, units
+            ex_units = '[length]**2'
+            if not cs[0].check(ex_units):
+                raise ValueError((
+                    f'Unexpected dimensionality in cross_sections units, '
+                    f'expected {ex_units}, got {str(cs[0].dimensionality)}'))
+            cs = [x.to('mbarn').magnitude for x in cs]*ureg('mbarn')
+        else:
+            cs = None
+
+        evec_weights = np.real(np.einsum('ijkl,ijkl->ijk',
+                                         self.eigenvectors,
+                                         np.conj(self.eigenvectors)))
+        crystal = self.crystal
+        for i in range(crystal.n_atoms):
+            dos = self._calculate_dos(dos_bins, mode_widths=mode_widths,
+                                      mode_widths_min=mode_widths_min,
+                                      mode_weights=evec_weights[:, :, i])
+            if cs is not None:
+                # Neutron weighted DOS is per atom of sample
+                avg_atom_mass = np.mean(crystal._atom_mass)
+                dos *= cs[i]*avg_atom_mass/crystal._atom_mass[i]
+            if i == 0:
+                all_dos_y_data = np.zeros((
+                    crystal.n_atoms, len(dos_bins) - 1))*dos.units
+            all_dos_y_data[i] = dos
+        metadata = {'line_data':
+            [{'species': symbol, 'index': idx}
+             for idx, symbol in enumerate(crystal.atom_type)]}
+        if weighting is not None:
+            metadata['weighting'] = weighting
+
+        return Spectrum1DCollection(
+            dos_bins, all_dos_y_data, metadata=metadata)
+
     def to_dict(self) -> Dict[str, Any]:
         """
         Convert to a dictionary. See QpointPhononModes.from_dict for
@@ -407,7 +544,7 @@ class QpointPhononModes(QpointFrequencies):
             self.crystal, self.qpts, self.frequencies, self.weights)
 
     @classmethod
-    def from_dict(cls: T, d: Dict[str, Any]) -> T:
+    def from_dict(cls: Type[T], d: Dict[str, Any]) -> T:
         """
         Convert a dictionary to a QpointPhononModes object
 
@@ -432,7 +569,7 @@ class QpointPhononModes(QpointFrequencies):
                    d['eigenvectors'], d['weights'])
 
     @classmethod
-    def from_json_file(cls: T, filename: str) -> T:
+    def from_json_file(cls: Type[T], filename: str) -> T:
         """
         Read from a JSON file. See QpointPhononModes.from_dict for
         required fields
@@ -446,7 +583,7 @@ class QpointPhononModes(QpointFrequencies):
                                    type_dict={'eigenvectors': np.complex128})
 
     @classmethod
-    def from_castep(cls: T, filename: str) -> T:
+    def from_castep(cls: Type[T], filename: str) -> T:
         """
         Reads precalculated phonon mode data from a CASTEP .phonon file
 
@@ -459,7 +596,7 @@ class QpointPhononModes(QpointFrequencies):
         return cls.from_dict(data)
 
     @classmethod
-    def from_phonopy(cls: T, path: str = '.',
+    def from_phonopy(cls: Type[T], path: str = '.',
                      phonon_name: str = 'band.yaml',
                      phonon_format: Optional[str] = None,
                      summary_name: str = 'phonopy.yaml') -> T:
